"""Simple database module.


Authors: Valentyn Stadnytskyi
Date created: 2019-11-21

Python Version: 2.7 and 3.7
"""
from logging import debug,info,warning,error

__version__ = "1.5.6" # Issue: WindowsError in remove(settings_file)
from pdb import pm
from threading import RLock
lock = RLock()

from os import getcwd

class DataBase():
    def __init__(self,root,name):
        """
        initializes the connection to the database.

        Parameters
        ----------
        root :: (string)
            path or environment variable
        name :: (string)
            name of the database

        Returns
        -------

        Examples
        --------
        >>> write()
        """
        self.filename, self.root, self.name = self.get_filename(root,name)
        self.database = self.read()

    def get_filename(self,root,name):
        """
        generates filename based on input root and name. There are few special keywords like $TEMPDIR, $LOCALDIR that will obtain the locations of temporary and local directories.

        Parameters
        ----------
        root :: (string)
            path or environment variable
        name :: (string)
            name of the database

        Returns
        -------
        filename :: (string)
            full path from the home directory, including the name of the file.

        Examples
        --------
        >>> filename = get_filename(root, 'name')
        """
        from platform import system
        if system() == 'Darwin':
            bracket = '/'
        elif system() == 'Windows':
            bracket = '\\'
        else:
            bracket = '/'

        if root == "$TEMPDIR":
            from tempfile import gettempdir
            root = gettempdir() + bracket +'SavedProperty' + bracket
        elif root == "$LOCALDIR" or root == "":
            from os import getcwd
            root = getcwd() + bracket+'SavedProperty'+ bracket
        else:
            root = root + bracket+ 'SavedProperty' + bracket
        filename = root + name + '_db.py'
        return filename, root, name

    def read(self):
        """
        reads the database file

        Parameters
        ----------

        Returns
        -------
        dictionary :: (dictionary)
            returns a dictionary of all data in the database file.

        Examples
        --------
        >>> data = read()
        """
        import yaml
        from os.path import exists
        if exists(self.filename):
            with lock:
                with open(self.filename) as handle:
                    try:
                        data = yaml.safe_load(handle.read())  # (2)
                        return data
                    except yaml.YAMLError:
                        return {}  # (3)
        else:
            return {}

    def write(self):
        """
        writes current dictionary saved in memory to the database file

        Parameters
        ----------

        Returns
        -------

        Examples
        --------
        >>> write()
        """
        import yaml
        data = self.database
        with lock:
            with open(self.filename, 'w') as handle:
                yaml.dump(data, handle)

    def sync(self):
        """
        reads the database file and updates the self.database variable accordingly.

        Parameters
        ----------

        Returns
        -------

        Examples
        --------
        >>> sync()
        """
        self.database = self.read()


    def close(self):  # (4)
        pass

class SavedProperty():
    def __init__(self,db,name,value):
        self.name = name
        self.db = db
        self.default_value = value

    def init(self):
        """
        wrapper that returns property object with get and set functions.

        Parameters
        ----------

        Returns
        -------

        Examples
        --------
        >>> from saved_property import DataBase, SavedProperty
        >>> db = DataBase(root = '', name = 'simpledb')
        >>> d = SavedProperty(db,'d',5.0).init()
        """
        return property(self.get,self.set)

    def set(self,name,value) :
        """
        setter for SavedProperty. sets value  in the database

        Parameters
        ----------
        name :: (string)
            name of the entry in the database.
        value :: (object)
            value saved to the database.

        Returns
        -------

        Examples
        --------
        >>>
        """
        self.db.database[self.name] = value
        self.db.write()

    def get(self,name):
        """
        getter for SavedProperty. returns saved value if it exists in the database, otherwise returns default_value

        Parameters
        ----------
        name :: (string)
            name of the entry in the database.

        Returns
        -------
        value :: (object)
            returns a value stored with the selected key in the database

        Examples
        --------
        >>>
        """
        if self.name in list(self.db.database.keys()):
            return self.db.database[self.name]
        else:
            return self.default_value



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
plotting functions
author: Valentyn Stadnytskyi
data: 2017 - Nov 17 2018

"""
__vesrion__ = '1.0.0'
from time import  time, sleep
import sys
if sys.version_info[0] == 2:
    from time import clock as perf_counter
else:
    from time import perf_counter
import platform

#def generate_plot(x,y):#
    #from matplotlib.figure import Figure
    #from matplotlib import pyplot
    #from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
    #figure = Figure(figsize=(8,3),dpi=80)
    #axis = figure.add_subplot(1,1,1)
    #axis.plot(x,y)
    #return axis

def generate_plot(x,y):
    from matplotlib.figure import Figure
    from matplotlib import pyplot as plt
    #from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
    figure = plt.figure()
    axis = figure.add_subplot(1,1,1)
    axis.plot(x,y)
    return figure

from ubcs_auxiliary import save_object, load_object


if __name__ == '__main__':
    from numpy import arange, random
    x = arange(0,100,1)
    y = random.random((100,))
    #save_object('plot.pltpkl',generate_plot(x,y))
    figure_loaded = load_object('plot.pltpkl')

    #figure_loaded.show()



def find(topdir, name=[], exclude=[]):
    """A list of files found starting at 'topdir' that match the patterns given
    by 'name', excluding those matching the patterns given by 'exclude'.
    returns a vector of integers on logarithmic scale starting from decade start, ending decade end with M per decade

    Parameters
    ----------
    topdir (string)
    name (list)
    exclude (list)

    Returns
    -------
    file_list (list)

    Examples
    --------
    >>> res = anfinrud_auxiliary.os.walk('anfinrud_auxiliary/')
    >>> for i in res: print(i[0])
            ...:
        anfinrud_auxiliary/
        anfinrud_auxiliary/tests
        anfinrud_auxiliary/tests/__pycache__
        anfinrud_auxiliary/__pycache__

    Comments:
    ---------

    Python 3.7.4 (v3.7.4:e09359112e, Jul  8 2019, 14:54:52)
    Type 'copyright', 'credits' or 'license' for more information
    IPython 7.8.0 -- An enhanced Interactive Python. Type '?' for help.
    In [1]: from time import time
    In [2]: import auxiliary
    In [3]: res = auxiliary.os.walk('/')
    In [4]: t1 = time(); lst = list(res); t2 = time(); print(t2-t1, len(lst))
    3.9815242290496826 1346

    Python 2.7.16 (v2.7.16:413a49145e, Mar  2 2019, 14:32:10)
    Type "copyright", "credits" or "license" for more information.
    IPython 5.8.0 -- An enhanced Interactive Python.
    In [1]: from time import time
    In [2]: import anfinrud_auxiliary
    In [3]: res = auxiliary.os.walk('')
    In [4]: t1 = time(); lst = list(res); t2 = time(); print(t2-t1, len(lst))
    (0.77646803855896, 1346)
    """

    def glob_to_regex(pattern):
        return "^"+pattern.replace(".", "\\.").replace("*", ".*").replace("?", ".")+"$"
    try:
        from scandir import walk
    except ImportError:
        from os import walk
    import re
    if type(name) == str:
        name = [name]
    if type(exclude) == str:
        exclude = [exclude]
    name = [re.compile(glob_to_regex(pattern)) for pattern in name]
    exclude = [re.compile(glob_to_regex(pattern)) for pattern in exclude]

    file_list = []
    for (directory, subdirs, files) in walk(topdir):
        for file in files:
            pathname = directory+"/"+file
            match = any([pattern.match(pathname) for pattern in name]) and\
                not any([pattern.match(pathname) for pattern in exclude])
            if match:
                file_list += [pathname]
    return file_list

def exclude():
    """Returns a list of patterns to exclude from a search. Add
    terms as required."""
    exclude = ['*/alignment*',
               '*/trash*',
               '*/_Archived*',
               '*/backup*',
               '*/Commissioning*',
               '*/Test*',
               '*/.AppleDouble*',
               '*LaserX*',
               '*LaserZ*',
               '*Copy*',
               '*._*',
               '.DS_Store']
    return exclude

def image_file_names_from_path(beamtime,path_name):
    """Returns image file names found under path_name. Typically used with
    'Reference_*() , which specifies which directories contain data for which
    zinger-free-statistics are to be acquired. The zinger-free-statistics
    include Imean and Ivar, which are used to construct UC_psi.npy."""
    from db_functions import find,exclude
    from numpy import sort
    data_dir,analysis_dir = data_dir_analysis_dir(beamtime)
    terms = ['*.mccd','*.rx']
    image_files = sort(find(data_dir+path_name, name=terms, exclude=exclude()))
    return image_files

def N_files_in_dir(folder = '', match = '*'):
    import os
    import fnmatch
    integer = len(fnmatch.filter(os.listdir(folder), match))
    return integer



from numpy import zeros, array_equal

from ubcs_auxiliary import save_object, load_object

def test_save_load(tmpdir):
    arr_in = zeros((10,5))
    str_in = 'this is string'
    dict_in = {'1':1,'2':2}
    list_in = [1,2,3,4]
    compound_in = {}
    compound_in['arr'] = arr_in
    compound_in['str'] = str_in
    compound_in['dict'] = dict_in
    compound_in['list'] = list_in

    # save to a file
    save_object(tmpdir+'arr.extension',arr_in)
    save_object(tmpdir+'str.extension',str_in)
    save_object(tmpdir+'dict.extension',dict_in)
    save_object(tmpdir+'list.extension',list_in)
    save_object(tmpdir+'compound.extension',compound_in)

    # Load to a file
    arr_out = load_object(tmpdir+'arr.extension')
    str_out = load_object(tmpdir+'str.extension')
    dict_out = load_object(tmpdir+'dict.extension')
    list_out = load_object(tmpdir+'list.extension')
    compound_out = load_object(tmpdir+'compound.extension')

    #run tests
    assert array_equal(arr_in, arr_out)
    assert str_in == str_out
    assert dict_in == dict_out
    assert list_in == list_out
    assert compound_in.keys() == compound_out.keys()
